#include <windows.h>
#include <mygx.h>

extern "C" {

#include "resource.h"
#include "../stdafx.h"
#include "../openttd.h"
#include "../functions.h"
#include "../variables.h"
#include "../debug.h"
#include "../string.h"
#include "../table/strings.h"
#include "../gfx.h"
#include "../mixer.h"
#include "../window.h"
#include "../gui.h"
#include <mmsystem.h>
#include "../hal.h"
#include <winnt.h>
#include <wininet.h>
//#include <io.h>
#include "fcntl.h"
#include "../network.h"
#include "UIHelper.h"

#define GETRAWFRAMEBUFFER 0x00020001
#define FORMAT_565		1
#define FORMAT_555		2
#define FORMAT_OTHER	3

typedef struct _RawFramebufferInfo
{
	WORD	wFormat;
	WORD	wBPP;
	VOID*	pFramePointer;
	int		cxStride;
	int		cyStride;
	int		cxPixels;
	int		cyPixels;
} RawFrameBufferInfo;

extern int WndProcGapiC(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
unsigned int _random_seeds[2][2];
extern unsigned long Run(void);
//extern LRESULT CALLBACK WndProcGapi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
HINSTANCE _inst;
byte *_gapi_video_mem;
unsigned short pal[256];
GXDisplayProperties g_gxdp;
GXKeyList gxKeys;
byte _dirkeys;
bool _right_button_down;
bool _right_button_clicked;
bool _ctrl_pressed;
bool _shift_pressed;
int _num_dirty_rects;
Rect screenrect;
unsigned short _cur_resolution[2];
RECT _dirty_rects[MAX_DIRTY_RECTS];
FiosItem *_fios_list;
int _fios_num;
int _saveload_mode;
RawFrameBufferInfo rfbi;
bool hires;
bool screenrotated = true;

void Win32GapiFullScreen(bool full_screen) 
{
	return; 
}

void Win32GapiSetDirKeys(void)
{
	_dirkeys =
		(GetAsyncKeyState(gxKeys.vkLeft)	<	0 ? 1 : 0) |
		(GetAsyncKeyState(gxKeys.vkUp)		<	0 ? 2 : 0) |
		(GetAsyncKeyState(gxKeys.vkRight)	<	0 ? 4 : 0) |
		(GetAsyncKeyState(gxKeys.vkDown)	<	0 ? 8 : 0);

	if( GetAsyncKeyState(gxKeys.vkA) < 0 )
	{
		_right_button_clicked = true;
		_right_button_down = true;
	}
	else
	{
		_right_button_clicked = false;
		_right_button_down = false;
	}
	
	_ctrl_pressed = (GetAsyncKeyState(gxKeys.vkC)<0);
	_shift_pressed = (GetAsyncKeyState(gxKeys.vkB)<0);

}

LRESULT CALLBACK WndProcGapi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) 
	{
	case WM_KILLFOCUS:
		GXSuspend();
		break;
	case WM_SETFOCUS:
		GXResume();
		break;
	case WM_KEYDOWN:
		Win32GapiSetDirKeys();
		break;
	case WM_KEYUP:
		Win32GapiSetDirKeys();
		break;
	default:
		WndProcGapiC(hwnd, msg, wParam, lParam);
		break;
	}
	
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS wc = {		CS_HREDRAW | CS_VREDRAW,
						(WNDPROC) WndProcGapi,
						0,
						0,
						hInstance,
						NULL,
						0,
						(HBRUSH) GetStockObject(BLACK_BRUSH),
						0,
						szWindowClass
						};

	return RegisterClass(&wc);
}

const char *Win32GapiStart(const char * const *parm)
{
	if( hires )
	{
		HDC hdc = GetDC(NULL);
		ExtEscape(	hdc, GETRAWFRAMEBUFFER, 0, NULL,
					sizeof(RawFrameBufferInfo), (char *) &rfbi);
		ReleaseDC(NULL, hdc);		
	}

	_screen.width = 640;
	_screen.height = 480;
	_screen.pitch = (_screen.width + 3) & ~0x3;

	_gapi_video_mem = (byte*) malloc(_screen.width*_screen.height);

	if(_gapi_video_mem == NULL )
	{
		MessageBox(NULL,TEXT("Not enough free ram for display!"),TEXT("ERROR"),MB_ICONEXCLAMATION);
	}

	// autodetect landscape or portrait mode
	if( g_HIDPI_Width >= g_HIDPI_Height ) screenrotated = false;
	else screenrotated = true;

	// now set to full screen
	if( screenrotated )
	{
		_cur_resolution[1] = g_HIDPI_Width;
		_cur_resolution[0] = g_HIDPI_Height;
		gxKeys = GXGetDefaultKeys(GX_LANDSCAPEKEYS);
	}
	else
	{
		_cur_resolution[0] = g_HIDPI_Width;
		_cur_resolution[1] = g_HIDPI_Height;
		gxKeys = GXGetDefaultKeys(GX_NORMALKEYS);
	}

	screenrect.left = (_screen.width / 2) - (_cur_resolution[0] / 2); // center
	if( hires ) screenrect.top = 0;
	else screenrect.top = 70;
	screenrect.right = screenrect.left + _cur_resolution[0];
	screenrect.bottom = screenrect.top + _cur_resolution[1];

	// boundry check
	if( screenrect.right > _screen.width ) screenrect.right = _screen.width;
	if( screenrect.bottom > _screen.height ) screenrect.bottom = _screen.height;

	_cursor.pos.x = screenrect.left + 1;
	_cursor.pos.y = screenrect.top;

	_num_dirty_rects = 0;
	
	return NULL;

}

void Win32GapiStop(void)
{
	GXCloseInput();
	GXCloseDisplay();
	free(_gapi_video_mem);
}

void Win32GapiMakeDirty(int left, int top, int width, int height)
{
	if (_num_dirty_rects < MAX_DIRTY_RECTS) 
	{
		_dirty_rects[_num_dirty_rects].left = left;
		_dirty_rects[_num_dirty_rects].right = left + width;
		_dirty_rects[_num_dirty_rects].top = top;
		_dirty_rects[_num_dirty_rects].bottom = top + height;
		
		_num_dirty_rects++;
	}

	return;
}

void Win32GapiDrawScreen(void)
{
	unsigned char *VidMem;
	int x, y, yStride, i;
	int xPitch, yPitch; 
	int iHeight = g_HIDPI_Height;
	int iWidth  = g_HIDPI_Width;
	byte *src;
	unsigned long address;

	if( _num_dirty_rects == 0 )
	{
		return;
	}

	if(hires)
	{
		VidMem = (unsigned char *)rfbi.pFramePointer;
		xPitch = rfbi.cxStride;
		yPitch = rfbi.cyStride;
	}
	else
	{
		VidMem = (unsigned char *)GXBeginDraw();
		xPitch = g_gxdp.cbxPitch;
		yPitch = g_gxdp.cbyPitch;
	}

	if( VidMem == NULL )
	{
		return;
	}
			
	if ( _num_dirty_rects >= MAX_DIRTY_RECTS )
	{
		if( true == screenrotated )
		{
			// rotated blit
			for( y = 0; y < iHeight; y++ )
			{
				yStride = (y * yPitch);
				for ( x = 0; x < iWidth; x++ )
				{
					address = (x * xPitch) + yStride;

					src = &_gapi_video_mem[_screen.width * (screenrect.top + x)];

					*(unsigned short *)(VidMem + address) = pal[ src[(screenrect.right - y)] ];
				}
			}
		}
		else
		{
			// normal blit
			for( y = 0; y < iHeight; y++ )
			{
				yStride = (y * yPitch);
				for ( x = 0; x < iWidth; x++ )
				{
					address = (x * xPitch) + yStride;

					src = &_gapi_video_mem[_screen.width * (screenrect.top + y)];

					*(unsigned short *)(VidMem + address) = pal[ src[screenrect.left + x] ];
				}
			}
		}
	}
	else
	{

		for(i = 0; i < _num_dirty_rects; i++)
		{
			Rect _clipped_rect;
			Rect _rotated_rect;
			_clipped_rect.top = _dirty_rects[i].top - screenrect.top;
			_clipped_rect.bottom = _clipped_rect.top + (_dirty_rects[i].bottom - _dirty_rects[i].top);
			_clipped_rect.left = _dirty_rects[i].left - screenrect.left;
			_clipped_rect.right = _clipped_rect.left + (_dirty_rects[i].right - _dirty_rects[i].left);
			
			// rotate clipped rect
			if( true == screenrotated )
			{
				// rotated
				_rotated_rect.top =		iHeight - _clipped_rect.right;
				_rotated_rect.bottom =	iHeight - _clipped_rect.left;
				_rotated_rect.left =	_clipped_rect.top;
				_rotated_rect.right =	_clipped_rect.bottom;
			}
			else
			{
				// not rotated
				_rotated_rect = _clipped_rect;
			}

			if( _rotated_rect.top >= iHeight ) _rotated_rect.top = iHeight - 1;
			if( _rotated_rect.top < 0 ) _rotated_rect.top = 0;
			
			if( _rotated_rect.bottom >= iHeight ) _rotated_rect.bottom = iHeight - 1;
			if( _rotated_rect.bottom < 0 ) _rotated_rect.bottom = 0;
			
			if( _rotated_rect.left > iWidth ) _rotated_rect.left = iWidth;
			if( _rotated_rect.left < 0 ) _rotated_rect.left = 0;
			
			if( _rotated_rect.right > iWidth ) _rotated_rect.right = iWidth;
			if( _rotated_rect.right < 0 ) _rotated_rect.right = 0;	

			if( true == screenrotated )
			{
				// rotate blit
				for( y = _rotated_rect.top; y <= _rotated_rect.bottom; y++ )
				{
					yStride = (y * yPitch);
					for ( x = _rotated_rect.left; x < _rotated_rect.right; x++ )
					{
						address = (x * xPitch) + yStride;

						src = &_gapi_video_mem[_screen.width * (screenrect.top + x)];

						*(unsigned short *)(VidMem + address) = pal[ src[(screenrect.right - y)] ];
					}
				}
			}
			else
			{
				// normal blit
				for( y = _rotated_rect.top; y <= _rotated_rect.bottom; y++ )
				{
					yStride = (y * yPitch);
					for ( x = _rotated_rect.left; x < _rotated_rect.right; x++ )
					{
						address = (x * xPitch) + yStride;

						src = &_gapi_video_mem[_screen.width * (screenrect.top + y)];

						*(unsigned short *)(VidMem + address) = pal[ src[screenrect.left + x] ];
					}
				}				
			}
		}
	}
	
	if(!hires)
	{
		GXEndDraw();
	}

	_num_dirty_rects = 0;
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPTSTR lpCmdLine, int nCmdShow)
{
	HWND hWnd;
	HBITMAP hBitmap;
	BITMAP bm;
	HDC myhdc;
	HDC hdcSource;

	TCHAR szTitle[] = _T("LOADING PPC TTD...");
	TCHAR szWindowClass[] = _T("LOADING PPC TTD...");

	hires = false;

	HIDPI_InitScaling();
	
	if( g_HIDPI_Width == 480 || g_HIDPI_Width == 640 )
	{
		hires = true;
	}
	
	//If it is already running, then focus on the window
	hWnd = FindWindow(szWindowClass, szTitle);	
	if (hWnd) 
	{
		SetForegroundWindow ((HWND) (((DWORD)hWnd) | 0x01));
		return 0;
	} 

	MyRegisterClass(hInstance, szWindowClass);
	
	hWnd =	CreateWindow(szWindowClass, szTitle, 
				WS_VISIBLE, 0, 0, 
				g_HIDPI_Width, 
				g_HIDPI_Height, 
				NULL, NULL, hInstance, NULL);

	if( !hWnd )
	{
		return FALSE;
	}

	// display the new window
	// MYCODE DEBUG: ShowWindow(hWnd, nCmdShow );
	UpdateWindow(hWnd);
	
	_inst = hInstance;

	_random_seeds[0][0] = GetTickCount();
	_random_seeds[0][1] = _random_seeds[0][0] * 0x1234567;

	// init GAPI
	if( GXOpenDisplay(hWnd, GX_FULLSCREEN) == 0 )
	{
		return FALSE;
	}

	// get display properties
	g_gxdp = GXGetDisplayProperties();

	// make sure 16 bit color is available
	if( !(g_gxdp.cBPP == 16) ||
		!(g_gxdp.ffFormat | kfDirect565) )
	{
		MessageBox(hWnd,
					_T("A Full 16-bit color display is required."),
					_T("Error:"), MB_ICONEXCLAMATION | MB_OK);
		GXCloseDisplay();
		return FALSE;
	}

	GXOpenInput();
	gxKeys = GXGetDefaultKeys(GX_LANDSCAPEKEYS);

/* EDIT Temp Debug:
	hBitmap = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_BITMAP4)); 
	myhdc = GetDC(NULL);
	hdcSource = CreateCompatibleDC(myhdc);
	SelectObject(hdcSource, hBitmap);
	GetObject(hBitmap, sizeof(BITMAP), &bm);
	BitBlt(myhdc, 0, 0, bm.bmWidth, bm.bmHeight, hdcSource, 0, 0, SRCCOPY);
	DeleteObject(hBitmap);
	DeleteDC(hdcSource);
	
	hBitmap = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_BITMAP1)); 
	myhdc = GetDC(NULL);
	hdcSource = CreateCompatibleDC(myhdc);
	SelectObject(hdcSource, hBitmap);
	GetObject(hBitmap, sizeof(BITMAP), &bm);
	BitBlt(myhdc, (g_HIDPI_Width / 2) - (bm.bmWidth / 2), (g_HIDPI_Height / 2) - (bm.bmHeight / 2), bm.bmWidth, bm.bmHeight, hdcSource, 0, 0, SRCCOPY);
	DeleteObject(hBitmap);
	DeleteDC(hdcSource);
	Sleep(2500);

	hBitmap = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_BITMAP4)); 
	myhdc = GetDC(NULL);
	hdcSource = CreateCompatibleDC(myhdc);
	SelectObject(hdcSource, hBitmap);
	GetObject(hBitmap, sizeof(BITMAP), &bm);
	BitBlt(myhdc, 0, 0, bm.bmWidth, bm.bmHeight, hdcSource, 0, 0, SRCCOPY);
	DeleteObject(hBitmap);
	DeleteDC(hdcSource);
	
	hBitmap = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_BITMAP3)); 
	myhdc = GetDC(NULL);
	hdcSource = CreateCompatibleDC(myhdc);
	SelectObject(hdcSource, hBitmap);
	GetObject(hBitmap, sizeof(BITMAP), &bm);
	BitBlt(myhdc, (g_HIDPI_Width / 2) - (bm.bmWidth / 2), (g_HIDPI_Height / 2) - (bm.bmHeight / 2), bm.bmWidth, bm.bmHeight, hdcSource, 0, 0, SRCCOPY);
	DeleteObject(hBitmap);
	DeleteDC(hdcSource);
*/
	// EDIT argc, argv = NULL, NULL
	//ttd_main(0, NULL);
	Run();
	
	return 0;
}

} // END of extern "C"
