#include "resource.h"
#include "../stdafx.h"
#include "../openttd.h"
#include "../functions.h"
#include "../variables.h"
#include "../debug.h"
#include "../string.h"
#include "../table/strings.h"
#include "../gfx.h"
#include "../mixer.h"
#include "../window.h"
#include "../gui.h"
#include <mmsystem.h>
#include "../hal.h"
#include <winnt.h>
#include <wininet.h>
//#include <io.h>
#include "fcntl.h"
#include "../network.h"

extern void DoExitSave(void);
extern byte *_gapi_video_mem;

extern void Win32GapiFullScreen(bool full_screen);
extern void Win32GapiStop(void);
extern const char *Win32GapiStart(const char * const *parm);
extern void Win32GapiDrawScreen(void);
extern void Win32GapiMakeDirty(int left, int top, int width, int height);

extern int _num_dirty_rects;
extern Rect screenrect;
extern HINSTANCE _inst;
extern bool hires;
extern int g_HIDPI_Width;
extern int g_HIDPI_Height;
extern bool screenrotated;

static struct {
	HWND main_wnd;
	HBITMAP dib_sect;
	void *bitmap_bits;
	void *buffer_bits;
	void *alloced_bits;
	HPALETTE gdi_palette;
	int width,height;
	int width_org, height_org;
	bool cursor_visible;
	bool switch_driver;
	bool fullscreen;
	bool double_size;
	bool has_focus;
	bool running;
} _wnd;

static bool MyShowCursor(bool show)
{
	if (_wnd.cursor_visible == show)
		return show;

	_wnd.cursor_visible = show;
	ShowCursor(show);

	return !show;
}

typedef struct {
	byte vk_from;
	byte vk_count;
	byte map_to;
} VkMapping;

#define AS(x, z) {x, 0, z}
#define AM(x, y, z, w) {x, y - x, z}

#ifndef	VK_OEM_3
#define VK_OEM_3 0xC0
#endif

static const VkMapping _vk_mapping[] = {
	// Pageup stuff + up/down
	AM(VK_PRIOR,VK_DOWN, WKC_PAGEUP, WKC_DOWN),
	// Map letters & digits
	AM('A','Z','A','Z'),
	AM('0','9','0','9'),

	AS(VK_ESCAPE,		WKC_ESC),
	AS(VK_PAUSE, WKC_PAUSE),
	AS(VK_BACK,			WKC_BACKSPACE),
	AM(VK_INSERT,VK_DELETE,WKC_INSERT, WKC_DELETE),

	AS(VK_SPACE,		WKC_SPACE),
	AS(VK_RETURN,		WKC_RETURN),
	AS(VK_TAB,			WKC_TAB),

	// Function keys
	AM(VK_F1, VK_F12,	WKC_F1, WKC_F12),

	// Numeric part.
	// What is the virtual keycode for numeric enter??
	AM(VK_NUMPAD0,VK_NUMPAD9, WKC_NUM_0, WKC_NUM_9),
	AS(VK_DIVIDE,			WKC_NUM_DIV),
	AS(VK_MULTIPLY,		WKC_NUM_MUL),
	AS(VK_SUBTRACT,		WKC_NUM_MINUS),
	AS(VK_ADD,				WKC_NUM_PLUS),
	AS(VK_DECIMAL,		WKC_NUM_DECIMAL)
};

static uint MapWindowsKey(uint sym)
{
	const VkMapping *map;
	uint key = 0;

	for (map = _vk_mapping; map != endof(_vk_mapping); ++map) {
		if ((uint)(sym - map->vk_from) <= map->vk_count) {
			key = sym - map->vk_from + map->map_to;
			break;
		}
	}

	if (GetAsyncKeyState(VK_SHIFT)<0)	key |= WKC_SHIFT;
	if (GetAsyncKeyState(VK_CONTROL)<0)	key |= WKC_CTRL;
	if (GetAsyncKeyState(VK_MENU)<0)	key |= WKC_ALT;
	return key;
}

static bool Win32GapiChangeRes(int w, int h)
{
	return false;
}

int WndProcGapiC(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) 
	{
	case WM_CLOSE:
		if (_game_mode == GM_MENU) { // do not ask to quit on the main screen
			_exit_game = true;
		} else if (_patches.autosave_on_exit) {
			DoExitSave();
			_exit_game = true;
		} else
			AskExitGame();

		return 0;

	case WM_LBUTTONDOWN:
	{
		POINT pt;
		int x, y, realx, realy;
		unsigned short screenchanged = 0;

		SetCapture(hwnd);

		_left_button_clicked = true;
		_left_button_down = true;

		// set x and y according to screen rotation
		realx = (int16)LOWORD(lParam);
		realy = (int16)HIWORD(lParam);
		x = g_HIDPI_Height - realy;
		y = realx;

		// jam the viewport position code here
		// check edges of screen for stylus input using rotated coords only if
		// the resolution of the game doesnt match the screen size
		if( (	g_HIDPI_Width	!= 640		&&
				g_HIDPI_Height	!= 480	)	||
			(	g_HIDPI_Height	!= 640		&&
				g_HIDPI_Width	!= 480	)	)
		{
			if( realy > (g_HIDPI_Height-10) )
			{
				screenrect.left -= 10;
				screenchanged = 1;
			}
			if( realy < 10 )
			{
		 		screenrect.left += 10;
		 		screenchanged = 1;
			}
			if( realx > (g_HIDPI_Width-10) )
			{
 				screenrect.top += 10;
		 		screenchanged = 1;
			}
			if( realx < 10 ) 	
			{
				screenrect.top -= 10;
		 		screenchanged = 1;
			}
			
			if( 1 == screenchanged )
			{
				// check security of screenrect
				if( screenrect.left < 0 ) 				screenrect.left = 0;
				if( screenrect.left > (_screen.width-_cur_resolution[0]) ) 	screenrect.left = _screen.width - _cur_resolution[0];
				if( screenrect.top < 0 ) 				screenrect.top = 0;
				if( screenrect.top > (_screen.height-_cur_resolution[1]) ) 	screenrect.top = _screen.height - _cur_resolution[1];
				
				screenrect.right = screenrect.left + _cur_resolution[0];
				screenrect.bottom = screenrect.top + _cur_resolution[1];
				
				// reposition the toolbar and status bar
				if (_game_mode != GM_MENU) 
				{
					PositionMainToolbar(NULL);
				}
				// make sure window is drawn
				MarkWholeScreenDirty();
				_num_dirty_rects = MAX_DIRTY_RECTS;
				
				_left_button_down = false;		
				_left_button_clicked = false;
			}
		}
		
		x += screenrect.left;
		y += screenrect.top;

		if (_cursor.fix_at) {
			int dx = x - _cursor.pos.x;
			int dy = y - _cursor.pos.y;
			if (dx != 0 || dy != 0) {
				_cursor.delta.x += dx;
				_cursor.delta.y += dy;

				pt.x = _cursor.pos.x;
				pt.y = _cursor.pos.y;

				ClientToScreen(hwnd, &pt);
				SetCursorPos(pt.x, pt.y);
			}
		} else {
			_cursor.delta.x = x - _cursor.pos.x;
			_cursor.delta.y = y - _cursor.pos.y;
			_cursor.pos.x = x;
			_cursor.pos.y = y;
			_cursor.dirty = true;
		}
		MyShowCursor(false);
		return 0;
	}

	case WM_LBUTTONUP:
	{
		POINT pt;
		int x, y;

		ReleaseCapture();
		
		_left_button_down = false;		
		_left_button_clicked = false;

		// set x and y according to screen rotation
		x = g_HIDPI_Height - (int16)HIWORD(lParam);
		y = (int16)LOWORD(lParam);

		x += screenrect.left;
		y += screenrect.top;

		if (_cursor.fix_at) {
			int dx = x - _cursor.pos.x;
			int dy = y - _cursor.pos.y;
			if (dx != 0 || dy != 0) {
				_cursor.delta.x += dx;
				_cursor.delta.y += dy;

				pt.x = _cursor.pos.x;
				pt.y = _cursor.pos.y;

				ClientToScreen(hwnd, &pt);
				SetCursorPos(pt.x, pt.y);
			}
		} else {
			_cursor.delta.x = x - _cursor.pos.x;
			_cursor.delta.y = y - _cursor.pos.y;
			_cursor.pos.x = x;
			_cursor.pos.y = y;
			_cursor.dirty = true;
		}
		MyShowCursor(false);
		return 0;
	}
	case WM_RBUTTONDOWN:
		SetCapture(hwnd);
		_right_button_down = true;
		_right_button_clicked = true;
		return 0;

	case WM_RBUTTONUP:
		ReleaseCapture();
		_right_button_down = false;
		return 0;

	case WM_MOUSEMOVE: 
	{
		POINT pt;

		int x, y, realx, realy;
		unsigned short screenchanged = 0;

		SetCapture(hwnd);

		//_left_button_down = true;
		_left_button_clicked = false;
		
		// set x and y according to screen rotation
		realx = (int16)LOWORD(lParam);
		realy = (int16)HIWORD(lParam);
		x = g_HIDPI_Height - realy;
		y = realx;
		
		// jam the viewport position code here
		// check edges of screen for stylus input using rotated coords only if
		// the resolution of the game doesnt match the screen size
		if( (	g_HIDPI_Width	!= 640		&&
				g_HIDPI_Height	!= 480	)	||
			(	g_HIDPI_Height	!= 640		&&
				g_HIDPI_Width	!= 480	)	)
		{
			if( realy > (g_HIDPI_Height-10) )
			{
				screenrect.left -= 10;
				screenchanged = 1;
			}
			if( realy < 10 )
			{
		 		screenrect.left += 10;
		 		screenchanged = 1;
			}
			if( realx > (g_HIDPI_Width-10) )
			{
 				screenrect.top += 10;
		 		screenchanged = 1;
			}
			if( realx < 10 ) 	
			{
				screenrect.top -= 10;
		 		screenchanged = 1;
			}
			
			if( 1 == screenchanged )
			{
				// check security of screenrect
				if( screenrect.left < 0 ) 				screenrect.left = 0;
				if( screenrect.left > (_screen.width-_cur_resolution[0]) ) 	screenrect.left = _screen.width - _cur_resolution[0];
				if( screenrect.top < 0 ) 				screenrect.top = 0;
				if( screenrect.top > (_screen.height-_cur_resolution[1]) ) 	screenrect.top = _screen.height - _cur_resolution[1];
				
				screenrect.right = screenrect.left + _cur_resolution[0];
				screenrect.bottom = screenrect.top + _cur_resolution[1];
				
				// reposition the toolbar and status bar
				if (_game_mode != GM_MENU) 
				{
					PositionMainToolbar(NULL);
				}
				// make sure window is drawn
				MarkWholeScreenDirty();
				_num_dirty_rects = MAX_DIRTY_RECTS;
				
				_left_button_down = false;		
				_left_button_clicked = false;
			}
		}

		x += screenrect.left;
		y += screenrect.top;

		if (_cursor.fix_at) {
			int dx = x - _cursor.pos.x;
			int dy = y - _cursor.pos.y;
			if (dx != 0 || dy != 0) {
				_cursor.delta.x += dx;
				_cursor.delta.y += dy;

				pt.x = _cursor.pos.x;
				pt.y = _cursor.pos.y;

				ClientToScreen(hwnd, &pt);
				SetCursorPos(pt.x, pt.y);
			}
		} else {
			_cursor.delta.x = x - _cursor.pos.x;
			_cursor.delta.y = y - _cursor.pos.y;
			_cursor.pos.x = x;
			_cursor.pos.y = y;
			_cursor.dirty = true;
		}
		MyShowCursor(false);
		return 0;
	}

	case WM_KEYDOWN: 
	{
		// this is the rewritten ascii input function
		// it disables windows deadkey handling --> more linux like :D
		unsigned short w = 0;
		int r = 0;
		//byte ks[256];
		unsigned int scan = 0;
		uint16 scancode = (( lParam & 0xFF0000 ) >> 16 );

		//GetKeyboardState(ks);
		//r = ToAscii(wParam, scan, ks, &w, 0);
		//if (r == 0) w = 0; // no translation was possible
		
		// MYCODE EDIT
		// takes care of ToAscii
		if( wParam >= 65 && wParam <= 90 )
			w = wParam;
		// checks for space as well
		if( wParam == 32 )
			w = wParam;
		
		_pressed_key = w | MapWindowsKey(wParam) << 16;

		if (scancode == 41)
			_pressed_key = w | WKC_BACKQUOTE << 16;
/*
		if ((_pressed_key >> 16) == ('D' | WKC_CTRL) && !_wnd.fullscreen) {
			_double_size ^= 1;
			_wnd.double_size = _double_size;
			ClientSizeChanged(_wnd.width, _wnd.height);
			MarkWholeScreenDirty();
		}
*/		
	}	break;

	case WM_SYSKEYDOWN: /* user presses F10 or Alt, both activating the title-menu */
		switch (wParam) {
		case VK_RETURN: case 0x46: /* Full Screen on ALT + ENTER/F(VK_F) */
			ToggleFullScreen(!_wnd.fullscreen);
			return 0;
		case VK_MENU: /* Just ALT */
			return 0; // do nothing
		case VK_F10: /* F10, ignore activation of menu */
			_pressed_key = MapWindowsKey(wParam) << 16;
			return 0;
		default: /* ALT in combination with something else */
			_pressed_key = MapWindowsKey(wParam) << 16;
			break;
		}
		break;
	
	case WM_ACTIVATE:
		_wnd.has_focus = (bool)wParam;
		break;
	}

	return 0;
}

extern unsigned short pal[256];

static void Win32GapiUpdatePalette(void)
{
	uint i;

	for (i = 0; i != 256; i++) 
	{
		pal[i] = (unsigned short)
				 (	((_cur_palette[i].r & 0xF8) << 8) |
					((_cur_palette[i].g & 0xFC) << 3) |
					((_cur_palette[i].b & 0xF8) >> 3) );
	}
}

static void Win32GapiMainLoop(void)
{
	MSG mesg;
	uint32 next_tick = GetTickCount() + 30, cur_ticks;
	uint32 pal_tick = 0;
	
	_wnd.running = true;

	while(true) {
		while (PeekMessage(&mesg, NULL, 0, 0, PM_REMOVE)) {
			InteractiveRandom(); // randomness
			TranslateMessage(&mesg);
			DispatchMessage(&mesg);
		}
		if (_exit_game)
		{
			return;
		}

#if defined(_DEBUG)
		if (_wnd.has_focus && GetAsyncKeyState(VK_SHIFT) < 0) {
			if (
#else
		if (_wnd.has_focus && GetAsyncKeyState(VK_TAB) < 0) {
			/* Disable speeding up game with ALT+TAB (if syskey is pressed, the
			 * real key is in the upper 16 bits (see WM_SYSKEYDOWN in WndProcGdi()) */
			if ((_pressed_key >> 16) & WKC_TAB &&
#endif
			    !_networking && _game_mode != GM_MENU)
				_fast_forward |= 2;
		} else if (_fast_forward & 2)
			_fast_forward = 0;

		cur_ticks = GetTickCount();
		if ((_fast_forward && !_pause) || cur_ticks > next_tick)
		{
			next_tick = cur_ticks;
		}

		if (cur_ticks == next_tick) {
			next_tick += 30;

			// don't scroll while not in PPC's focus
			if (!_wnd.has_focus) 
			{
				_dirkeys = 0;
			}

			GameLoop();
			_cursor.delta.x = _cursor.delta.y = 0;

			_screen.dst_ptr = _gapi_video_mem;
			UpdateWindows();
			if (++pal_tick > 4) 
			{
				if(_pal_last_dirty != -1) 
				{
					Win32GapiUpdatePalette();
					_pal_last_dirty = -1;
					_num_dirty_rects = MAX_DIRTY_RECTS;
				}

				pal_tick = 1;
			}

			Win32GapiDrawScreen();
		} else 
		{
			Sleep(1);
			//GdiFlush();
			_screen.dst_ptr = _gapi_video_mem;
			DrawTextMessage();
			DrawMouseCursor();
			if(_pal_last_dirty != -1) 
			{
				Win32GapiUpdatePalette();
				_pal_last_dirty = -1;
				_num_dirty_rects = MAX_DIRTY_RECTS;
			}

			Win32GapiDrawScreen();
		}
	}
}

const HalVideoDriver _gapi_video_driver = {
	Win32GapiStart,
	Win32GapiStop,
	Win32GapiMakeDirty,
	Win32GapiMainLoop,
	Win32GapiChangeRes,
	Win32GapiFullScreen,
};
