/* $Id: sdl_s.c 3552 2006-02-05 09:56:04Z tron $ */

#include "../stdafx.h"

#ifdef WITH_SDL

#include "../openttd.h"
#include "../driver.h"
#include "../mixer.h"
#include "../sdl_c.h"
#include "sdl_s.h"
#include <SDL.h>

#ifdef __SYMBIAN32__
static uint32 _mus_pos = 0;
struct music_block
{
	char* data_buffer;
	uint32 data_len;
	struct music_block* next_block;
	bool in_use;
};

static struct music_block* _music_block = NULL;
static struct music_block* _last_block = NULL;

struct music_block _music_blocks[2];
void add_block(char* aDataBuffer, int aLen)
{
	struct music_block* new_block = &_music_blocks[0];

	if(new_block->in_use)
	{
		new_block = &_music_blocks[1];
	}
	
	new_block->in_use = true;
	if(new_block->data_buffer == NULL)
	{
		new_block->data_buffer = malloc(aLen);
		new_block->data_len = aLen;
	}

	memcpy(new_block->data_buffer, aDataBuffer, aLen);
	new_block->next_block = NULL;

	if(_music_block == NULL)
	{
		_music_block = new_block;
		_last_block = _music_block;
		_mus_pos = 0;
	}
	else
	{
	_last_block->next_block = new_block;
	_last_block = new_block;
	}
}
extern void block_played();
#endif

static void CDECL fill_sound_buffer(void *userdata, Uint8 *stream, int len)
{
	MxMixSamples(stream, len / 4);

#ifdef __SYMBIAN32__
	{
		unsigned int pos = 0;
		while (_music_block != NULL && pos<len)
		{
			
			
			signed long int sample = *(signed short int*)(stream+pos)+(*((signed short int*)(_music_block->data_buffer+_mus_pos))>>1);
			*(signed short int*)(stream+pos) = sample;
			
			pos+=2;		
			
			_mus_pos+=2;
			if(_mus_pos == _music_block->data_len)
			{
				_music_block->in_use = false;					
				_music_block = _music_block->next_block;		
				_mus_pos = 0;
				block_played();	
			}
			
			sample = *(signed short int*)(stream+pos)+(*((signed short int*)(_music_block->data_buffer+_mus_pos))>>1);
			*(signed short int*)(stream+pos) = sample;
			_mus_pos+=2;
			pos+=2;
			if(_mus_pos == _music_block->data_len)
			{			
				_music_block->in_use = false;					
				_music_block = _music_block->next_block;		
				_mus_pos = 0;
				block_played();	
			}
		}
	}
#endif

}

static const char *SdlSoundStart(const char * const *parm)
{
	SDL_AudioSpec spec;
	const char *s = SdlOpen(SDL_INIT_AUDIO);
	if (s != NULL) return s;
	_music_blocks[0].data_buffer = NULL;
	_music_blocks[0].data_len = 0;
	_music_blocks[0].next_block = NULL;
	_music_blocks[0].in_use = false;
	_music_blocks[1].data_buffer = NULL;
	_music_blocks[1].data_len = 0;
	_music_blocks[1].next_block = NULL;
	_music_blocks[1].in_use = false;

	spec.freq = GetDriverParamInt(parm, "hz", 11025);
	spec.format = AUDIO_S16SYS;
	spec.channels = 2;
	spec.samples = 512;
	spec.callback = fill_sound_buffer;
	spec.userdata = NULL;
	SDL_CALL SDL_OpenAudio(&spec, &spec);
	SDL_CALL SDL_PauseAudio(0);
	return NULL;
}

static void SdlSoundStop(void)
{
	SDL_CALL SDL_CloseAudio();
	SdlClose(SDL_INIT_AUDIO);
	free(_music_blocks[0].data_buffer);
	free(_music_blocks[1].data_buffer);
}

const HalSoundDriver _sdl_sound_driver = {
	SdlSoundStart,
	SdlSoundStop,
};

#endif
