#ifndef ____SYMBIAN32__
extern "C"
{
#include "../stdafx.h"
#include "openttd.h"
#include "variables.h"
#include "ttdstring.h"
#include "../hal.h"
#include "../sound.h"
}
#include "extmidi.h"
//#include "fcntl.h"
#include "../../TimidityCE\config.h"
#include "../../TimidityCE\common.h"
#include "../../TimidityCE\instrum.h"
#include "../../TimidityCE\playmidi.h"
#include "../../TimidityCE\readmidi.h"
#include "../../TimidityCE\output.h"
#include "../../TimidityCE\tables.h"
#include "../../TimidityCE\myprintf.h"
#include "../../TimidityCE\errno.h"

#define MAX_WIDE_STR	1024

char song[MAX_PATH];
TUint32 threadId;

int free_instruments_afterwards = 1;
// MYCODE EDIT
static char def_instr_name[256] = "Patch\\acpiano.pat";
//ControlMode ctl;
extern MidiEvent *current_event;

MIDICONTROL _midi;

RThread gMusicThread;
#define MAXWORDS 10

extern "C" char *GetPath()
{       
	return _path.personal_dir;
} 

static int
read_config_file (char* name)
{
	char* error="Error!";
	FILE *fp;
	char  *w[MAXWORDS], *cp;
	char buffer[1024];
	ToneBank *bank = 0;
	int i, j, k, line = 0, words;
	static int rcf_count = 0;
	char strWide[MAX_WIDE_STR];

	if (rcf_count > 50) {
		myprintf ("Error!","Probable source loop in configuration files");
		return (-1);
    }
	
	sprintf(strWide, "%s%s", GetPath(), name);

	if (!(fp = open_file (strWide, 1, OF_VERBOSE)))
		return -1;
	
	while (fgets (buffer, 1024, fp)) {
		
		line++;
		w[words = 0] = strtok (buffer, " \t\r\n\240");
		if (!w[0] || (*w[0] == '#')) {
			continue;
		}
		while (w[words] && (words < MAXWORDS)) {
			w[++words] = strtok (0, " \t\r\n\240");
		}
		if (!strcmp (w[0], "dir")) {
			if (words < 2) {
				myprintf (error,"%s: line %d: No directory given\n", name, line);
				return -2;
			}
			for (i = 1; i < words; i++)
				add_to_pathlist (w[i]);
		} else if (!strcmp (w[0], "source")) {
			if (words < 2) {
				myprintf (error, "%s: line %d: No file name given\n", name, line);
				return -2;
			}
			for (i = 1; i < words; i++) 
			{
				rcf_count++;
				sprintf(strWide, "%s%s", GetPath(), w[i]);
				read_config_file (strWide);
				rcf_count--;
			}
		} else if (!strcmp (w[0], "default")) {
			if (words != 2)	{
				myprintf(error,"%s: line %d: Must specify exactly one patch name\n",
					name, line);
				return -2;
			}
			strncpy (def_instr_name, w[1], 255);
			def_instr_name[255] = '\0';
		} else if (!strcmp (w[0], "drumset")) {
			if (words < 2) {
				myprintf(error,"%s: line %d: No drum set number given\n",
					name, line);
				return -2;
			}
			i = atoi (w[1]);
			if (i < 0 || i > 127) {
				myprintf (error,
					"%s: line %d: Drum set must be between 0 and 127\n",
					name, line);
				return -2;
			}
			if (!drumset[i]) {
				drumset[i] = (ToneBank *)safe_malloc (sizeof (ToneBank));
				memset (drumset[i], 0, sizeof (ToneBank));
			}
			bank = drumset[i];
		}
		else if (!strcmp (w[0], "bank")) {
			if (words < 2) {
				myprintf(error,"%s: line %d: No bank number given\n",
					name, line);
				return -2;
			}
			i = atoi (w[1]);
			if (i < 0 || i > 127) {
				myprintf(error,
					"%s: line %d: Tone bank must be between 0 and 127\n",
					name, line);
				return -2;
			}
			if (!tonebank[i]) {
				tonebank[i] = (ToneBank *)safe_malloc (sizeof (ToneBank));
				memset (tonebank[i], 0, sizeof (ToneBank));
			}
			bank = tonebank[i];
		} else {
			if ((words < 2) || (*w[0] < '0' || *w[0] > '9')) {
				myprintf (error,"%s: line %d: syntax error\n", name, line);
				return -2;
			}
			i = atoi (w[0]);
			if (i < 0 || i > 127) {
				myprintf (error,
					"%s: line %d: Program must be between 0 and 127\n",
					name, line);
				return -2;
			}
			if (!bank) {
				myprintf (error,
					"%s: line %d: Must specify tone bank or drum set "
					"before assignment\n", name, line);
				return -2;
			}
			if (bank->tone[i].name) {
				free (bank->tone[i].name);
			}
			strcpy ((bank->tone[i].name = (char*)safe_malloc(2*(strlen(w[1])+1))),w[1]);
			bank->tone[i].note = bank->tone[i].amp = bank->tone[i].pan =
				bank->tone[i].strip_loop = bank->tone[i].strip_envelope =
				bank->tone[i].strip_tail = -1;
			
			for (j = 2; j < words; j++) {
				if (!(cp = strchr (w[j], '='))) {
					myprintf (error, "%s: line %d: bad patch option %s\n",
						name, line, w[j]);
					return -2;
				}
				*cp++ = 0;
				if (!strcmp (w[j], "amp")) {
					k = atoi (cp);
					if ((k < 0 || k > MAX_AMPLIFICATION)
						|| (*cp < '0' || *cp > '9')) {
						myprintf (error,
							"%s: line %d: amplification must be between "
							"0 and %d\n", name, line, MAX_AMPLIFICATION);
						return -2;
					}
					bank->tone[i].amp = k;
				} else if (!strcmp (w[j], "note")) {
					k = atoi (cp);
					if ((k < 0 || k > 127) || (*cp < '0' || *cp > '9'))	{
						myprintf (error,
							"%s: line %d: note must be between 0 and 127\n",
							name, line);
						return -2;
					}
					bank->tone[i].note = k;
				} else if (!strcmp (w[j], "pan")) {
					if (!strcmp (cp, "center"))
						k = 64;
					else if (!strcmp (cp, "left"))
						k = 0;
					else if (!strcmp (cp, "right"))
						k = 127;
					else
						k = ((atoi (cp) + 100) * 100) / 157;
					if ((k < 0 || k > 127) ||
						(k == 0 && *cp != '-' && (*cp < '0' || *cp > '9')))	{
						myprintf (error,
							"%s: line %d: panning must be left, right, "
							"center, or between -100 and 100\n",
							name, line);
						return -2;
					}
					bank->tone[i].pan = k;
				} else if (!strcmp (w[j], "keep")) {
					if (!strcmp (cp, "env"))
						bank->tone[i].strip_envelope = 0;
					else if (!strcmp (cp, "loop"))
						bank->tone[i].strip_loop = 0;
					else {
						myprintf (error,
							"%s: line %d: keep must be env or loop\n",
							name, line);
						return -2;
					}
				} else if (!strcmp (w[j], "strip")) {
					if (!strcmp (cp, "env"))
						bank->tone[i].strip_envelope = 1;
					else if (!strcmp (cp, "loop"))
						bank->tone[i].strip_loop = 1;
					else if (!strcmp (cp, "tail"))
						bank->tone[i].strip_tail = 1;
					else {
						myprintf (error,
							"%s: line %d: strip must be env, loop, or tail\n",
							name, line);
						return -2;
					}
				} else {
					myprintf(error, "%s: line %d: bad patch option %s\n",
						name, line, w[j]);
					return -2;
				}
			}
		}
    }
	if (ferror (fp)) {	
		const char* text = getError(errno);
		
		myprintf(error, "Can't read %s: %s\n", name, text);
		close_file (fp);
		LocalFree(text);
		return -2;
    }
	close_file (fp);
	return 0;
}

extern "C" void ExtMidiStop(void)
{
	if (_midi.playing) 
	{
		_midi.stop_song = true;
		_midi.start_song[0] = '\0';
	}
	
	_midi.terminate = true;
	gMusicThread.Resume();
	gMusicThread.Close();
	play_mode->close_output();
}

extern "C" void ExtMidiPlaySong(const char* filename)
{
	strcpy(_midi.start_song, filename);

	if(_midi.playing)
	{
	_midi.stop_song = true;
	}
	else
	{
	_midi.stop_song = false;
	}

	_midi.playing = true;
	gMusicThread.Resume();
}

extern "C" void ExtMidiStopSong(void)
{
	if (_midi.playing) 
	{
		_midi.stop_song = true;
		_midi.playing = false;
		_midi.start_song[0] = '\0';
	}
}

extern "C" bool ExtMidiIsPlaying(void)
{
	return _midi.playing;
}

extern "C" void ExtMidiSetVolume(byte vol)
{
	_midi.new_vol = vol;
}

static TInt  ExtMidiGo(TAny* /*aArg*/)
{
	RThread musicThread;
	do {
		char *s;
		s = _midi.start_song;
		if (s[0] != '\0') 
		{			
			play_midi_file(s);
		
			// Delay somewhat in case we don't manage to play.
			if (!_midi.playing) 
			{
				// MYCODE EDIT 5000 to 1000
				s[0] = '\0';			
			}
		}
		else
		{
			User::After(250000);
		}
		
		if (_midi.stop_song && _midi.playing) 
		{
			_midi.stop_song = false;
			_midi.playing = false;
			ExtMidiStopSong();
		}

		if (_midi.playing && !ExtMidiIsPlaying())
			_midi.playing = false;

		if(!_midi.playing && !_midi.terminate)
			musicThread.Suspend();
	} while (!_midi.terminate);
	musicThread.Close();

	return 0;
}

extern "C" const char* ExtMidiStart(const char* const * parm)
{
	int32 output_rate = DEFAULT_RATE;	
	int buffer_fragments = -1;
	int got_a_configuration = 0;
	memset(&_midi, 0, sizeof(_midi));
	_midi.new_vol = -1;
	_midi.playing = false;
	song[0] = '\0';	

	read_config_file(CONFIG_FILE);
		
	//output_rate = 11025;
	antialiasing_allowed=1;
	buffer_fragments = 2;		/* On Win32 2 is chunky */

	if (!got_a_configuration)
    {
		//MessageBox(GetActiveWindow(), Cannot read timidity.cfg!", "Error!", MB_ICONSTOP);
		//assert(0);
		//exit(1);
    }
	
	/* Set play mode parameters */
	play_mode->rate = output_rate;
	if (buffer_fragments != -1)
		play_mode->extra_param[0] = buffer_fragments;
	
	
	init_tables ();
		
	/* Open output device */
	if (play_mode->open_output() < 0)
    {
		myprintf ("Error!", "Couldn't open %s\n", play_mode->id_name);	
    }
		
	if (!control_ratio)
    {
		control_ratio = play_mode->rate / CONTROLS_PER_SECOND;
		if (control_ratio < 1)
			control_ratio = 1;
		else if (control_ratio > MAX_CONTROL_RATIO)
			control_ratio = MAX_CONTROL_RATIO;
    }

/*	if (*def_instr_name)
	{
		set_default_instrument (def_instr_name);
	}*/
	
//	InitializeCriticalSection (&critSect);
	gMusicThread.Create(_L("OTDDMusicThread"),ExtMidiGo, 8192, NULL, NULL);
	gMusicThread.Resume();

	return NULL;
}

#endif /* __SYMBIAN32__ */
